@@ -30,27 +30,49 @@
 #include <gdk/gdkkeysyms.h>
 #endif
 
+#include "eggaccelerators.h"
+
 #include "gsd-keygrab.h"
 
-/* we exclude shift, GDK_CONTROL_MASK and GDK_MOD1_MASK since we know what
-   these modifiers mean
-   these are the mods whose combinations are bound by the keygrabbing code */
-#define GSD_IGNORED_MODS (0x2000 /*Xkb modifier*/ | GDK_LOCK_MASK | GDK_HYPER_MASK)
+/* these are the mods whose combinations are ignored by the keygrabbing code */
+static GdkModifierType gsd_ignored_mods = 0;
 
 /* these are the ones we actually use for global keys, we always only check
  * for these set */
-#define GSD_USED_MODS (GDK_SHIFT_MASK | GDK_CONTROL_MASK |\
-		       GDK_MOD1_MASK | GDK_MOD2_MASK | GDK_MOD3_MASK | GDK_MOD4_MASK |\
-		       GDK_MOD5_MASK | GDK_SUPER_MASK | GDK_META_MASK)
+static GdkModifierType gsd_used_mods = 0;
 
+static void
+setup_modifiers (void)
+{
+        if (gsd_used_mods == 0 || gsd_ignored_mods == 0) {
+                GdkModifierType dynmods;
 
-static gboolean
+                /* default modifiers */
+                gsd_ignored_mods = \
+                        0x2000 /*Xkb modifier*/ | GDK_LOCK_MASK | GDK_HYPER_MASK;
+		gsd_used_mods = \
+                        GDK_SHIFT_MASK | GDK_CONTROL_MASK |\
+                        GDK_MOD1_MASK | GDK_MOD2_MASK | GDK_MOD3_MASK | GDK_MOD4_MASK |\
+                        GDK_MOD5_MASK | GDK_SUPER_MASK | GDK_META_MASK;
+
+                /* NumLock can be assigned to varying keys so we need to
+                 * resolve and ignore it specially */
+                dynmods = 0;
+                egg_keymap_resolve_virtual_modifiers (gdk_keymap_get_default (),
+                                                      EGG_VIRTUAL_NUM_LOCK_MASK,
+                                                      &dynmods);
+
+                gsd_ignored_mods |= dynmods;
+                gsd_used_mods &= ~dynmods;
+	}
+}
+
+static void
 grab_key_real (guint      keycode,
                GdkWindow *root,
                gboolean   grab,
                int        mask)
 {
-        gdk_error_trap_push ();
         if (grab) {
                 XGrabKey (GDK_DISPLAY (),
                           keycode,
@@ -65,28 +87,44 @@ grab_key_real (guint      keycode,
                             mask,
                             GDK_WINDOW_XID (root));
         }
-
-        gdk_flush ();
-        return gdk_error_trap_pop () == 0;
 }
 
 /* Grab the key. In order to ignore GSD_IGNORED_MODS we need to grab
  * all combinations of the ignored modifiers and those actually used
  * for the binding (if any).
  *
- * inspired by all_combinations from gnome-panel/gnome-panel/global-keys.c */
+ * inspired by all_combinations from gnome-panel/gnome-panel/global-keys.c
+ *
+ * This may generate X errors.  The correct way to use this is like:
+ *
+ *        gdk_error_trap_push ();
+ *
+ *        grab_key_unsafe (key, grab, screens);
+ *
+ *        gdk_flush ();
+ *        if (gdk_error_trap_pop ())
+ *                g_warning ("Grab failed, another application may already have access to key '%u'",
+ *                           key->keycode);
+ *
+ * This is not done in the function itself, to allow doing multiple grab_key
+ * operations with one flush only.
+ */
 #define N_BITS 32
 void
-grab_key (Key                 *key,
-          gboolean             grab,
-          GSList              *screens)
+grab_key_unsafe (Key                 *key,
+                 gboolean             grab,
+                 GSList              *screens)
 {
         int   indexes[N_BITS]; /* indexes of bits we need to flip */
         int   i;
         int   bit;
         int   bits_set_cnt;
         int   uppervalue;
-        guint mask = GSD_IGNORED_MODS & ~key->state & GDK_MODIFIER_MASK;
+        guint mask;
+
+        setup_modifiers ();
+
+        mask = gsd_ignored_mods & ~key->state & GDK_MODIFIER_MASK;
 
         bit = 0;
         /* store the indexes of all set bits in mask in the array */
@@ -114,14 +152,10 @@ grab_key (Key                 *key,
 
                 for (l = screens; l; l = l->next) {
                         GdkScreen *screen = l->data;
-                        if (!grab_key_real (key->keycode,
-                                            gdk_screen_get_root_window (screen),
-                                            grab,
-                                            result | key->state)) {
-                                g_warning ("Grab failed, another application may already have access to key '%u'",
-                                           key->keycode);
-                                return;
-                        }
+                        grab_key_real (key->keycode,
+                                       gdk_screen_get_root_window (screen),
+                                       grab,
+                                       result | key->state);
                 }
         }
 }
@@ -162,6 +196,8 @@ match_key (Key *key, XEvent *event)
 	if (key == NULL)
 		return FALSE;
 
+	setup_modifiers ();
+
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
 	if (have_xkb (event->xkey.display))
 		group = XkbGroupForCoreState (event->xkey.state);
@@ -184,11 +220,11 @@ match_key (Key *key, XEvent *event)
 			consumed &= ~GDK_SHIFT_MASK;
 
 		return ((lower == key->keysym || upper == key->keysym)
-			&& (event->xkey.state & ~consumed & GSD_USED_MODS) == key->state);
+			&& (event->xkey.state & ~consumed & gsd_used_mods) == key->state);
 	}
 
 	/* The key we passed doesn't have a keysym, so try with just the keycode */
         return (key != NULL
                 && key->keycode == event->xkey.keycode
-                && key->state == (event->xkey.state & GSD_USED_MODS));
+                && key->state == (event->xkey.state & gsd_used_mods));
 }
