@@ -255,6 +255,7 @@ static void
 binding_register_keys (GsdKeybindingsManager *manager)
 {
         GSList *li;
+        gboolean need_flush = FALSE;
 
         gdk_error_trap_push ();
 
@@ -266,10 +267,11 @@ binding_register_keys (GsdKeybindingsManager *manager)
                     binding->previous_key.state != binding->key.state) {
                         /* Ungrab key if it changed and not clashing with previously set binding */
                         if (! key_already_used (manager, binding)) {
+                                need_flush = TRUE;
                                 if (binding->previous_key.keycode) {
-                                        grab_key (&binding->previous_key, FALSE, manager->priv->screens);
+                                        grab_key_unsafe (&binding->previous_key, FALSE, manager->priv->screens);
                                 }
-                                grab_key (&binding->key, TRUE, manager->priv->screens);
+                                grab_key_unsafe (&binding->key, TRUE, manager->priv->screens);
 
                                 binding->previous_key.keysym = binding->key.keysym;
                                 binding->previous_key.state = binding->key.state;
@@ -278,8 +280,11 @@ binding_register_keys (GsdKeybindingsManager *manager)
                                 g_warning ("Key binding (%s) is already in use", binding->binding_str);
                 }
         }
-        gdk_flush ();
-        gdk_error_trap_pop ();
+
+        if (need_flush)
+                gdk_flush ();
+        if (gdk_error_trap_pop ())
+                g_warning ("Grab failed for some keys, another application may already have access the them.");
 }
 
 extern char **environ;
@@ -453,7 +458,7 @@ register_config_callback (GsdKeybindingsManager   *manager,
                           const char              *path,
                           GConfClientNotifyFunc    func)
 {
-        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
+        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
         return gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
@@ -472,6 +477,13 @@ gsd_keybindings_manager_start (GsdKeybindingsManager *manager,
         g_debug ("Starting keybindings manager");
         gnome_settings_profile_start (NULL);
 
+        client = gconf_client_get_default ();
+
+        manager->priv->notify = register_config_callback (manager,
+                                                          client,
+                                                          GCONF_BINDING_DIR,
+                                                          (GConfClientNotifyFunc) bindings_callback);
+
         dpy = gdk_display_get_default ();
         screen_num = gdk_display_get_n_screens (dpy);
 
@@ -482,13 +494,6 @@ gsd_keybindings_manager_start (GsdKeybindingsManager *manager,
                                        manager);
         }
 
-        client = gconf_client_get_default ();
-
-        manager->priv->notify = register_config_callback (manager,
-                                                          client,
-                                                          GCONF_BINDING_DIR,
-                                                          (GConfClientNotifyFunc) bindings_callback);
-
         list = gconf_client_all_dirs (client, GCONF_BINDING_DIR, NULL);
         manager->priv->screens = get_screens_list ();
 
