@@ -38,7 +38,6 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
-#define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
@@ -59,15 +58,6 @@
 
 #define VOLUME_STEP 6           /* percents for one volume button press */
 
-/* we exclude shift, GDK_CONTROL_MASK and GDK_MOD1_MASK since we know what
-   these modifiers mean
-   these are the mods whose combinations are bound by the keygrabbing code */
-#define IGNORED_MODS (0x2000 /*Xkb modifier*/ | GDK_LOCK_MASK  | \
-       GDK_MOD2_MASK | GDK_MOD3_MASK | GDK_MOD4_MASK | GDK_MOD5_MASK)
-/* these are the ones we actually use for global keys, we always only check
- * for these set */
-#define USED_MODS (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)
-
 #define GSD_MEDIA_KEYS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_MEDIA_KEYS_MANAGER, GsdMediaKeysManagerPrivate))
 
 typedef struct {
@@ -88,6 +78,7 @@ struct GsdMediaKeysManagerPrivate
         GList           *media_players;
 
         DBusGConnection *connection;
+        guint            notify[HANDLED_KEYS];
 };
 
 enum {
@@ -113,15 +104,6 @@ init_screens (GsdMediaKeysManager *manager)
         int i;
 
         display = gdk_display_get_default ();
-        if (gdk_display_get_n_screens (display) == 1) {
-                GdkScreen *screen;
-
-                screen = gdk_screen_get_default ();
-                manager->priv->screens = g_slist_append (manager->priv->screens, screen);
-                manager->priv->current_screen = screen;
-                return;
-        }
-
         for (i = 0; i < gdk_display_get_n_screens (display); i++) {
                 GdkScreen *screen;
 
@@ -132,7 +114,7 @@ init_screens (GsdMediaKeysManager *manager)
                 manager->priv->screens = g_slist_append (manager->priv->screens, screen);
         }
 
-        manager->priv->current_screen = (GdkScreen *)manager->priv->screens->data;
+        manager->priv->current_screen = manager->priv->screens->data;
 }
 
 
@@ -145,11 +127,11 @@ acme_error (char * msg)
                                                GTK_DIALOG_MODAL,
                                                GTK_MESSAGE_ERROR,
                                                GTK_BUTTONS_OK,
-                                               "%s", msg);
+                                               msg, NULL);
         gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
                                          GTK_RESPONSE_OK);
         gtk_widget_show (error_dialog);
-        g_signal_connect (G_OBJECT (error_dialog),
+        g_signal_connect (error_dialog,
                           "response",
                           G_CALLBACK (gtk_widget_destroy),
                           NULL);
@@ -265,7 +247,7 @@ dialog_init (GsdMediaKeysManager *manager)
 {
         if (manager->priv->dialog != NULL
             && !gsd_media_keys_window_is_valid (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog))) {
-                g_object_unref (manager->priv->dialog);
+                gtk_widget_destroy (manager->priv->dialog);
                 manager->priv->dialog = NULL;
         }
 
@@ -274,80 +256,6 @@ dialog_init (GsdMediaKeysManager *manager)
         }
 }
 
-static gboolean
-grab_key_real (Key       *key,
-               GdkWindow *root,
-               gboolean   grab,
-               int        result)
-{
-        gdk_error_trap_push ();
-        if (grab) {
-                g_debug ("Grab: %d %d %x %s", result, (int)key->keycode, key->state, XKeysymToString (key->keysym));
-                XGrabKey (GDK_DISPLAY (),
-                          key->keycode,
-                          (result | key->state),
-                          GDK_WINDOW_XID (root),
-                          True,
-                          GrabModeAsync,
-                          GrabModeAsync);
-        } else {
-                g_debug ("UnGrab: %d %d %x", result, (int)key->keycode, key->state);
-                XUngrabKey (GDK_DISPLAY (),
-                            key->keycode,
-                            (result | key->state),
-                            GDK_WINDOW_XID (root));
-        }
-
-        gdk_flush ();
-
-        gdk_error_trap_pop ();
-
-        return TRUE;
-}
-
-/* inspired from all_combinations from gnome-panel/gnome-panel/global-keys.c */
-#define N_BITS 32
-static void
-grab_key (GsdMediaKeysManager *manager,
-          Key                 *key,
-          gboolean             grab)
-{
-        int   indexes[N_BITS];/*indexes of bits we need to flip*/
-        int   i;
-        int   bit;
-        int   bits_set_cnt;
-        int   uppervalue;
-        guint mask_to_traverse = IGNORED_MODS & ~ key->state;
-
-        bit = 0;
-        for (i = 0; i < N_BITS; i++) {
-                if (mask_to_traverse & (1 << i)) {
-                        indexes[bit++] = i;
-                }
-        }
-
-        bits_set_cnt = bit;
-
-        uppervalue = 1 << bits_set_cnt;
-        for (i = 0; i < uppervalue; i++) {
-                GSList *l;
-                int     j;
-                int     result = 0;
-
-                for (j = 0; j < bits_set_cnt; j++) {
-                        if (i & (1 << j)) {
-                                result |= (1 << indexes[j]);
-                        }
-                }
-
-                for (l = manager->priv->screens; l ; l = l->next) {
-                        GdkScreen *screen = l->data;
-                        if (grab_key_real (key, gdk_screen_get_root_window (screen), grab, result) == FALSE)
-                                return;
-                }
-        }
-}
-
 static gboolean
 is_valid_shortcut (const char *string)
 {
@@ -368,20 +276,21 @@ update_kbd_cb (GConfClient         *client,
                GsdMediaKeysManager *manager)
 {
         int      i;
-        gboolean found = FALSE;
+        gboolean need_flush = TRUE;
 
         g_return_if_fail (entry->key != NULL);
 
+        gdk_error_trap_push ();
+
         /* Find the key that was modified */
         for (i = 0; i < HANDLED_KEYS; i++) {
                 if (strcmp (entry->key, keys[i].gconf_key) == 0) {
                         char *tmp;
                         Key  *key;
 
-                        found = TRUE;
-
                         if (keys[i].key != NULL) {
-                                grab_key (manager, keys[i].key, FALSE);
+                                need_flush = TRUE;
+                                grab_key_unsafe (keys[i].key, FALSE, manager->priv->screens);
                         }
 
                         g_free (keys[i].key);
@@ -403,7 +312,8 @@ update_kbd_cb (GConfClient         *client,
                                 break;
                         }
 
-                        grab_key (manager, key, TRUE);
+                        need_flush = TRUE;
+                        grab_key_unsafe (key, TRUE, manager->priv->screens);
                         keys[i].key = key;
 
                         g_free (tmp);
@@ -412,32 +322,38 @@ update_kbd_cb (GConfClient         *client,
                 }
         }
 
-        if (found != FALSE) {
-                return;
-        }
+        if (need_flush)
+                gdk_flush ();
+        if (gdk_error_trap_pop ())
+                g_warning ("Grab failed for some keys, another application may already have access the them.");
 }
 
 static void
 init_kbd (GsdMediaKeysManager *manager)
 {
         int i;
+        gboolean need_flush = FALSE;
 
         gnome_settings_profile_start (NULL);
 
+        gdk_error_trap_push ();
+
         for (i = 0; i < HANDLED_KEYS; i++) {
                 char *tmp;
                 Key  *key;
 
-                gconf_client_notify_add (manager->priv->conf_client,
-                                         keys[i].gconf_key,
-                                         (GConfClientNotifyFunc)update_kbd_cb,
-                                         manager,
-                                         NULL,
-                                         NULL);
+                manager->priv->notify[i] =
+                        gconf_client_notify_add (manager->priv->conf_client,
+                                                 keys[i].gconf_key,
+                                                 (GConfClientNotifyFunc) update_kbd_cb,
+                                                 manager,
+                                                 NULL,
+                                                 NULL);
 
                 tmp = gconf_client_get_string (manager->priv->conf_client,
                                                keys[i].gconf_key,
                                                NULL);
+
                 if (!is_valid_shortcut (tmp)) {
                         g_debug ("Not a valid shortcut: '%s'", tmp);
                         g_free (tmp);
@@ -453,7 +369,7 @@ init_kbd (GsdMediaKeysManager *manager)
                         g_free (key);
                         continue;
                 }
-        /*avoid grabbing all the keyboard when KeyCode cannot be retrieved */
+                /* avoid grabbing all the keyboard when KeyCode cannot be retrieved */
                 if (key->keycode == AnyKey)  {
                         g_warning ("The shortcut key \"%s\" cannot be found on the current system, ignoring the binding", tmp);
                         g_free (tmp);
@@ -465,9 +381,15 @@ init_kbd (GsdMediaKeysManager *manager)
 
                 keys[i].key = key;
 
-                grab_key (manager, key, TRUE);
+                need_flush = TRUE;
+                grab_key_unsafe (key, TRUE, manager->priv->screens);
         }
 
+        if (need_flush)
+                gdk_flush ();
+        if (gdk_error_trap_pop ())
+                g_warning ("Grab failed for some keys, another application may already have access the them.");
+
         gnome_settings_profile_end (NULL);
 }
 
@@ -654,7 +576,7 @@ do_www_action (GsdMediaKeysManager *manager,
 static void
 do_exit_action (GsdMediaKeysManager *manager)
 {
-        execute (manager, "gnome-session-save --kill", FALSE, FALSE);
+        execute (manager, "gnome-session-save --shutdown-dialog", FALSE, FALSE);
 }
 
 static void
@@ -686,6 +608,7 @@ do_sound_action (GsdMediaKeysManager *manager,
         gboolean muted;
         int      vol;
         int      vol_step;
+        GError  *error = NULL;
 
         if (manager->priv->volume == NULL) {
                 return;
@@ -693,10 +616,17 @@ do_sound_action (GsdMediaKeysManager *manager,
 
         vol_step = gconf_client_get_int (manager->priv->conf_client,
                                          GCONF_MISC_DIR "/volume_step",
-                                         NULL);
+                                         &error);
 
-        if (vol_step == 0) {
+        if (error) {
                 vol_step = VOLUME_STEP;
+                g_error_free (error);
+        }
+
+        if (vol_step > 0) {
+                int threshold = acme_volume_get_threshold (manager->priv->volume);
+                if (vol_step < threshold)
+                        vol_step = threshold;
         }
 
         /* FIXME: this is racy */
@@ -947,27 +877,16 @@ acme_filter_events (GdkXEvent           *xevent,
                     GsdMediaKeysManager *manager)
 {
         XEvent    *xev = (XEvent *) xevent;
-        XAnyEvent *xanyev = (XAnyEvent *) xevent;
-        guint      keycode;
-        guint      state;
+        XAnyEvent *xany = (XAnyEvent *) xevent;
         int        i;
 
         /* verify we have a key event */
-        if (xev->xany.type != KeyPress
-            && xev->xany.type != KeyRelease) {
+        if (xev->type != KeyPress && xev->type != KeyRelease) {
                 return GDK_FILTER_CONTINUE;
         }
 
-        keycode = xev->xkey.keycode;
-        state = xev->xkey.state;
-
         for (i = 0; i < HANDLED_KEYS; i++) {
-                if (keys[i].key == NULL) {
-                        continue;
-                }
-
-                if (keys[i].key->keycode == keycode
-                    && (state & USED_MODS) == keys[i].key->state) {
+                if (match_key (keys[i].key, xev)) {
                         switch (keys[i].key_type) {
                         case VOLUME_DOWN_KEY:
                         case VOLUME_UP_KEY:
@@ -982,7 +901,7 @@ acme_filter_events (GdkXEvent           *xevent,
                                 }
                         }
 
-                        manager->priv->current_screen = acme_get_screen_from_event (manager, xanyev);
+                        manager->priv->current_screen = acme_get_screen_from_event (manager, xany);
 
                         if (do_action (manager, keys[i].key_type) == FALSE) {
                                 return GDK_FILTER_REMOVE;
@@ -1040,21 +959,64 @@ void
 gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
 {
         GsdMediaKeysManagerPrivate *priv = manager->priv;
+        GSList *ls;
+        GList *l;
+        int i;
 
         g_debug ("Stopping media_keys manager");
 
+        for (ls = priv->screens; ls != NULL; ls = ls->next) {
+                gdk_window_remove_filter (gdk_screen_get_root_window (ls->data),
+                                          (GdkFilterFunc) acme_filter_events,
+                                          manager);
+        }
+
+        g_slist_free (priv->screens);
+        priv->screens = NULL;
+
         if (priv->conf_client) {
+                gconf_client_remove_dir (priv->conf_client,
+                                         GCONF_BINDING_DIR,
+                                         NULL);
+
+                for (i = 0; i < HANDLED_KEYS; ++i) {
+                        if (priv->notify[i] != 0) {
+                                gconf_client_notify_remove (priv->conf_client, priv->notify[i]);
+                                priv->notify[i] = 0;
+                        }
+                }
+
                 g_object_unref (priv->conf_client);
                 priv->conf_client = NULL;
         }
 
+        if (priv->connection != NULL) {
+                dbus_g_connection_unref (priv->connection);
+                priv->connection = NULL;
+        }
+
+        for (i = 0; i < HANDLED_KEYS; ++i) {
+                g_free (keys[i].key);
+                keys[i].key = NULL;
+        }
+
         if (priv->volume) {
                 g_object_unref (priv->volume);
                 priv->volume = NULL;
         }
 
-        g_slist_free (priv->screens);
-        priv->screens = NULL;
+        if (priv->dialog != NULL) {
+                gtk_widget_destroy (priv->dialog);
+                priv->dialog = NULL;
+        }
+
+        for (l = priv->media_players; l; l = l->next) {
+                MediaPlayer *mp = l->data;
+                g_free (mp->application);
+                g_free (mp);
+        }
+        g_list_free (priv->media_players);
+        priv->media_players = NULL;
 }
 
 static void
@@ -1174,14 +1136,13 @@ register_manager (GsdMediaKeysManager *manager)
 {
         GError *error = NULL;
 
-        error = NULL;
         manager->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
         if (manager->priv->connection == NULL) {
                 if (error != NULL) {
-                        g_critical ("error getting session bus: %s", error->message);
+                        g_error ("Error getting session bus: %s", error->message);
                         g_error_free (error);
                 }
-                exit (1);
+                return FALSE;
         }
 
         dbus_g_connection_register_g_object (manager->priv->connection, GSD_MEDIA_KEYS_DBUS_PATH, G_OBJECT (manager));
