@@ -692,6 +692,7 @@ gsd_xrandr_manager_start (GsdXrandrManager *manager,
                           GError          **error)
 {
         g_debug ("Starting xrandr manager");
+        gnome_settings_profile_start (NULL);
 
         manager->priv->rw_screen = gnome_rr_screen_new (
                 gdk_screen_get_default (), on_randr_event, manager);
@@ -707,7 +708,7 @@ gsd_xrandr_manager_start (GsdXrandrManager *manager,
         g_assert (manager->priv->notify_id == 0);
 
         gconf_client_add_dir (manager->priv->client, CONF_DIR,
-                              GCONF_CLIENT_PRELOAD_NONE,
+                              GCONF_CLIENT_PRELOAD_ONELEVEL,
                               NULL);
 
         manager->priv->notify_id =
@@ -745,6 +746,8 @@ gsd_xrandr_manager_start (GsdXrandrManager *manager,
 
         start_or_stop_icon (manager);
 
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
