@@ -208,6 +208,7 @@ static TranslationEntry translations [] = {
         { "/desktop/gnome/interface/icon_theme",             "Net/IconThemeName",       GCONF_VALUE_STRING,   translate_string_string },
         { "/desktop/gnome/interface/file_chooser_backend",   "Gtk/FileChooserBackend",  GCONF_VALUE_STRING,   translate_string_string },
         { "/desktop/gnome/interface/menus_have_icons",       "Gtk/MenuImages",          GCONF_VALUE_BOOL,     translate_bool_int },
+        { "/desktop/gnome/interface/buttons_have_icons",     "Gtk/ButtonImages",          GCONF_VALUE_BOOL,     translate_bool_int },
         { "/desktop/gnome/interface/menubar_accel",          "Gtk/MenuBarAccel",        GCONF_VALUE_STRING,   translate_string_string },
         { "/desktop/gnome/peripherals/mouse/cursor_theme",   "Gtk/CursorThemeName",     GCONF_VALUE_STRING,   translate_string_string },
         { "/desktop/gnome/peripherals/mouse/cursor_size",    "Gtk/CursorThemeSize",     GCONF_VALUE_INT,      translate_int_int },
@@ -424,6 +425,7 @@ child_watch_cb (GPid     pid,
 {
         char *command = user_data;
 
+        gnome_settings_profile_end ("%s", command);
         if (!WIFEXITED (status) || WEXITSTATUS (status)) {
                 g_warning ("Command %s failed", command);
         }
@@ -446,6 +448,7 @@ spawn_with_input (const char *command,
                 return;
         }
 
+        gnome_settings_profile_start ("%s", command);
         error = NULL;
         res = g_spawn_async_with_pipes (NULL,
                                         argv,
@@ -563,6 +566,38 @@ fontconfig_callback (fontconfig_monitor_handle_t *handle,
         gnome_settings_profile_end (NULL);
 }
 
+static gboolean
+start_fontconfig_monitor_idle_cb (GnomeXSettingsManager *manager)
+{
+        gnome_settings_profile_start (NULL);
+
+        manager->priv->fontconfig_handle = fontconfig_monitor_start ((GFunc) fontconfig_callback, manager);
+
+        gnome_settings_profile_end (NULL);
+
+        return FALSE;
+}
+
+static void
+start_fontconfig_monitor (GnomeXSettingsManager  *manager)
+{
+        gnome_settings_profile_start (NULL);
+
+        fontconfig_cache_update ();
+
+        g_idle_add ((GSourceFunc) start_fontconfig_monitor_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+}
+
+static void
+stop_fontconfig_monitor (GnomeXSettingsManager  *manager)
+{
+        if (manager->priv->fontconfig_handle) {
+                fontconfig_monitor_stop (manager->priv->fontconfig_handle);
+                manager->priv->fontconfig_handle = NULL;
+        }
+}
 #endif /* HAVE_FONTCONFIG */
 
 static const char *
@@ -747,7 +782,6 @@ register_config_callback (GnomeXSettingsManager  *manager,
                           const char             *path,
                           GConfClientNotifyFunc   func)
 {
-        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
         return gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
@@ -824,6 +858,13 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
 
         client = gconf_client_get_default ();
 
+        gconf_client_add_dir (client, MOUSE_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+        gconf_client_add_dir (client, GTK_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+        gconf_client_add_dir (client, INTERFACE_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+        gconf_client_add_dir (client, SOUND_SETTINGS_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+        gconf_client_add_dir (client, GTK_MODULES_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+        gconf_client_add_dir (client, FONT_RENDER_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
         for (i = 0; i < G_N_ELEMENTS (translations); i++) {
                 GConfValue *val;
                 GError     *err;
@@ -876,7 +917,7 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                                           (GConfClientNotifyFunc) xft_callback);
         update_xft_settings (manager, client);
 
-        manager->priv->fontconfig_handle = fontconfig_monitor_start ((GFunc) fontconfig_callback, manager);
+        start_fontconfig_monitor (manager);
 #endif /* HAVE_FONTCONFIG */
 
         g_object_unref (client);
@@ -923,8 +964,7 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 #ifdef HAVE_FONTCONFIG
         gconf_client_remove_dir (client, FONT_RENDER_DIR, NULL);
 
-        fontconfig_monitor_stop (manager->priv->fontconfig_handle);
-        manager->priv->fontconfig_handle = NULL;
+        stop_fontconfig_monitor (manager);
 #endif /* HAVE_FONTCONFIG */
 
         for (i = 0; i < G_N_ELEMENTS (p->notify); ++i) {
